"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAvailableLocales = void 0;
var enums_1 = require("../enums");
var modules_1 = require("../modules");
var logger = new modules_1.Logger(modules_1.LogLevel.INFO, 'helpers/misc.ts');
/**
 * This method fetches and aggregate available locale from navigation service.
 * The {@link AvailableLocaleParams AvailableLocaleParams} object defines options to specify your request.
 * Example call:
 *
 * ```typescript
  const availableLocales = await getAvailableLocales({
    navigationServiceURL: 'https://your.navigation-service.url/navigation',
    projectId: 'xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx',
    contentMode: 'preview',
  })
 * ```
 * @param navigationServiceURL the url of navigation service
 * @param projectId the project id to use
 * @param contentMode the content mode to use, either 'preview' or 'release'
 * @returns an array of available locales in ISO format. e.g. ['en_GB', 'de_DE']
 */
var getAvailableLocales = function (_a) {
    var navigationServiceURL = _a.navigationServiceURL, projectId = _a.projectId, contentMode = _a.contentMode;
    return __awaiter(void 0, void 0, void 0, function () {
        var queryParam, url, navigationResponse, _b, message, ignore_1, data, locales_1, error_1;
        return __generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    _c.trys.push([0, 10, , 11]);
                    queryParam = contentMode + "." + projectId;
                    url = navigationServiceURL + "?from=" + queryParam + "&until=" + queryParam;
                    return [4 /*yield*/, fetch(url, {
                            method: 'GET',
                            headers: {
                                'Accept-Language': 'www',
                            },
                        })];
                case 1:
                    navigationResponse = _c.sent();
                    if (!!navigationResponse.ok) return [3 /*break*/, 8];
                    _b = navigationResponse.status;
                    switch (_b) {
                        case 401: return [3 /*break*/, 2];
                    }
                    return [3 /*break*/, 3];
                case 2: throw new Error(enums_1.FSXAApiErrors.NOT_AUTHORIZED);
                case 3:
                    if (!(navigationResponse.status === 400)) return [3 /*break*/, 7];
                    _c.label = 4;
                case 4:
                    _c.trys.push([4, 6, , 7]);
                    return [4 /*yield*/, navigationResponse.json()];
                case 5:
                    message = (_c.sent()).message;
                    logger.error("[getAvailableLocales] Bad Request: " + message);
                    return [3 /*break*/, 7];
                case 6:
                    ignore_1 = _c.sent();
                    return [3 /*break*/, 7];
                case 7: throw new Error(enums_1.FSXAApiErrors.UNKNOWN_ERROR);
                case 8: return [4 /*yield*/, navigationResponse.json()];
                case 9:
                    data = _c.sent();
                    if (!data._embedded || data._embedded.length === 0) {
                        return [2 /*return*/, []];
                    }
                    locales_1 = [];
                    data._embedded.forEach(function (item) {
                        var identifier = item.languageId;
                        if (identifier && !locales_1.includes(identifier)) {
                            locales_1.push(identifier);
                        }
                    });
                    return [2 /*return*/, locales_1];
                case 10:
                    error_1 = _c.sent();
                    logger.error("[getAvailableLocales] Error: " + error_1);
                    return [2 /*return*/, []];
                case 11: return [2 /*return*/];
            }
        });
    });
};
exports.getAvailableLocales = getAvailableLocales;
//# sourceMappingURL=misc.js.map