"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getMappedFilters = exports.UNKNOWN_ROUTE_ERROR = void 0;
var express_1 = __importDefault(require("express"));
var routes_1 = require("../routes");
var modules_1 = require("./../modules");
var enums_1 = require("../enums");
var endpointIntegrationWrapper_1 = require("./endpointIntegrationWrapper");
var LoggerChalked_1 = require("../modules/LoggerChalked");
exports.UNKNOWN_ROUTE_ERROR = 'Could not map given route and method.';
/**
 * Handles the resultObject from the wrapper, be it an error or a success.
 * On Success, it passes the fetched data, on error it uses the indicated
 * status Code and Error message to send an appropriate response.
 *
 * @param res response
 * @param result Wrapper Result
 * @returns Sends a response as indicated by the wrapper
 */
var sendWrapperResult = function (res, result) {
    if (result.success) {
        return res.json(result.data);
    }
    else {
        return res.status(result.error.statusCode).send({
            message: result.error.message,
            error: result.error.message,
        });
    }
};
/**
 * This handles Errors, that have not been accounted for by the wrapper.
 * Normally, this should not happen and the cause should be investaigated
 *
 * @param res response
 * @param req request
 * @param error caught error
 * @param logger logger
 * @returns sends an 500 internal server Error using the response.
 */
var sendUnexpectedError = function (res, req, error, logger) {
    logger.error("[" + req.originalUrl + "]", 'Unexpected error: ', error, req.body);
    return res.status(500).send({ message: error.message, error: error.message });
};
function getExpressRouter(_a) {
    var _this = this;
    var api = _a.api;
    var router = express_1.default.Router();
    var logger = new LoggerChalked_1.LoggerChalked(api.logLevel, 'Express-Server');
    var wrappers = (0, endpointIntegrationWrapper_1.useEndpointIntegrationWrapper)(api, 'Express-Server');
    router.use(express_1.default.json());
    router.post([routes_1.FETCH_ELEMENT_ROUTE, enums_1.FSXAProxyRoutes.FETCH_ELEMENT_ROUTE], function (req, res) { return __awaiter(_this, void 0, void 0, function () {
        var result, err_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    logger.info('Received POST on route: ', routes_1.FETCH_ELEMENT_ROUTE);
                    logger.debug('POST request body', req.body);
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, wrappers.fetchElementWrapper(req.body)];
                case 2:
                    result = _a.sent();
                    return [2 /*return*/, sendWrapperResult(res, result)];
                case 3:
                    err_1 = _a.sent();
                    sendUnexpectedError(res, req, err_1, logger);
                    return [3 /*break*/, 4];
                case 4: return [2 /*return*/];
            }
        });
    }); });
    router.post([routes_1.FETCH_NAVIGATION_ROUTE, enums_1.FSXAProxyRoutes.FETCH_NAVIGATION_ROUTE], function (req, res) { return __awaiter(_this, void 0, void 0, function () {
        var result, err_2;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    logger.info('Received POST on route', routes_1.FETCH_NAVIGATION_ROUTE);
                    logger.debug('POST request body', req.body);
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, wrappers.fetchNavigationWrapper(req.body)];
                case 2:
                    result = _a.sent();
                    return [2 /*return*/, sendWrapperResult(res, result)];
                case 3:
                    err_2 = _a.sent();
                    sendUnexpectedError(res, req, err_2, logger);
                    return [3 /*break*/, 4];
                case 4: return [2 /*return*/];
            }
        });
    }); });
    router.post([routes_1.FETCH_BY_FILTER_ROUTE, enums_1.FSXAProxyRoutes.FETCH_BY_FILTER_ROUTE], function (req, res) { return __awaiter(_this, void 0, void 0, function () {
        var result, err_3;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    logger.info('Received POST on route', routes_1.FETCH_BY_FILTER_ROUTE);
                    logger.debug('POST request body', req.body);
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, wrappers.fetchByFilterWrapper(req.body)];
                case 2:
                    result = _a.sent();
                    return [2 /*return*/, sendWrapperResult(res, result)];
                case 3:
                    err_3 = _a.sent();
                    sendUnexpectedError(res, req, err_3, logger);
                    return [3 /*break*/, 4];
                case 4: return [2 /*return*/];
            }
        });
    }); });
    router.post(enums_1.FSXAProxyRoutes.FETCH_PROPERTIES_ROUTE, function (req, res) { return __awaiter(_this, void 0, void 0, function () {
        var result, err_4;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    logger.info('Received POST on route', enums_1.FSXAProxyRoutes.FETCH_PROPERTIES_ROUTE);
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, wrappers.fetchProjectPropertiesWrapper(req.body)];
                case 2:
                    result = _a.sent();
                    return [2 /*return*/, sendWrapperResult(res, result)];
                case 3:
                    err_4 = _a.sent();
                    sendUnexpectedError(res, req, err_4, logger);
                    return [3 /*break*/, 4];
                case 4: return [2 /*return*/];
            }
        });
    }); });
    if (api.enableEventStream()) {
        router.get([routes_1.STREAM_CHANGE_EVENTS_ROUTE, enums_1.FSXAProxyRoutes.STREAM_CHANGE_EVENTS_ROUTE], (0, modules_1.eventStreamHandler)(api));
    }
    router.all('*', function (req, res) {
        logger.info('Requested unknown route', req.route);
        return res.json({
            error: exports.UNKNOWN_ROUTE_ERROR,
        });
    });
    return router;
}
exports.default = getExpressRouter;
var getMappedFilters = function (filters) {
    return (Array.isArray(filters)
        ? filters
        : filters
            ? [filters]
            : []);
};
exports.getMappedFilters = getMappedFilters;
//# sourceMappingURL=express.js.map