"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FSXARemoteApi = void 0;
var qs_1 = require("qs");
var _1 = require(".");
var utils_1 = require("../utils");
var enums_1 = require("./../enums");
var Logger_1 = require("./Logger");
var MappingUtils_1 = require("./MappingUtils");
var QueryBuilder_1 = require("./QueryBuilder");
var exceptions_1 = require("../exceptions");
var LoggerChalked_1 = require("./LoggerChalked");
/**
 * This class represents the `remote` variant of the FSXA API.
 */
var FSXARemoteApi = /** @class */ (function () {
    /**
     * The constructor of this class initializes the configuration for the api.
     *
     * @param config {@link FSXARemoteApiConfig FSXARemoteApiConfig}
     * @param config.apikey
     * @param config.caasURL
     * @param config.navigationServiceURL
     * @param config.tenantID
     * @param config.projectID
     * @param config.remotes optional {@link RemoteProjectConfiguration RemoteProjectConfiguration}
     * @param config.contentMode 'release' | 'preview'
     * @param config.maxReferenceDepth optional number to define the maximum depth of resolved objects
     * @param config.customMapper optional {@link CustomMapper CustomMapper}
     * @param config.filterOptions optional {@link RemoteApiFilterOptions RemoteApiFilterOptions} (EXPERIMENTAL)
     * @param config.logLevel the used {@link LogLevel LogLevel} for the API `(default LogLevel.ERROR)` - optional
     */
    function FSXARemoteApi(_a) {
        var apikey = _a.apikey, caasURL = _a.caasURL, navigationServiceURL = _a.navigationServiceURL, tenantID = _a.tenantID, projectID = _a.projectID, remotes = _a.remotes, contentMode = _a.contentMode, maxReferenceDepth = _a.maxReferenceDepth, customMapper = _a.customMapper, filterOptions = _a.filterOptions, _b = _a.logLevel, logLevel = _b === void 0 ? Logger_1.LogLevel.ERROR : _b;
        this.mode = 'remote';
        this._apikey = this.apikey;
        this._caasURL = this.caasURL;
        this._navigationServiceURL = this.navigationServiceURL;
        this._tenantID = this.tenantID;
        this._projectID = this.projectID;
        this._remotes = this.remotes;
        this._contentMode = this.contentMode;
        this._enableEventStream = false;
        this.apikey = apikey;
        this.caasURL = caasURL;
        this.navigationServiceURL = navigationServiceURL;
        this.tenantID = tenantID;
        this.projectID = projectID;
        this.remotes = remotes || {};
        this.contentMode = contentMode;
        this._maxReferenceDepth = maxReferenceDepth;
        this._customMapper = customMapper;
        this._logLevel = logLevel;
        this._logger = new LoggerChalked_1.LoggerChalked(logLevel, 'FSXARemoteApi');
        this._queryBuilder = new QueryBuilder_1.QueryBuilder(this._logger);
        this._navigationItemFilter = filterOptions === null || filterOptions === void 0 ? void 0 : filterOptions.navigationItemFilter;
        this._caasItemFilter = filterOptions === null || filterOptions === void 0 ? void 0 : filterOptions.caasItemFilter;
        this._logger.debug('FSXARemoteApi created', {
            caasURL: caasURL,
            navigationServiceURL: navigationServiceURL,
            tenantID: tenantID,
            projectID: projectID,
            remotes: remotes,
            contentMode: contentMode,
            customMapper: this._customMapper,
            navigationItemFilter: this._navigationItemFilter,
            caasItemFilter: this._caasItemFilter,
        });
    }
    Object.defineProperty(FSXARemoteApi.prototype, "authorizationHeader", {
        /**
         * Can be used in CaaS requests.
         * @returns an object with the configured apikey as value for the authorization key.
         */
        get: function () {
            return {
                authorization: "Bearer " + this.apikey,
            };
        },
        enumerable: false,
        configurable: true
    });
    FSXARemoteApi.prototype.verifyRemoteProjectExists = function (remoteProjectId) {
        var remoteProjectConfig = Object.values(this._remotes);
        var foundRemoteProject = remoteProjectConfig.find(function (config) { return config.id === remoteProjectId; });
        if (!foundRemoteProject) {
            throw new exceptions_1.HttpError(enums_1.FSXAApiErrors.UNKNOWN_REMOTE, enums_1.HttpStatus.NOT_FOUND);
        }
    };
    FSXARemoteApi.prototype.getRemoteConfigById = function (remoteProjectId) {
        var remoteProjectConfig = Object.values(this._remotes);
        var foundRemoteProject = remoteProjectConfig.find(function (config) { return config.id === remoteProjectId; });
        if (!foundRemoteProject) {
            throw new exceptions_1.HttpError(enums_1.FSXAApiErrors.UNKNOWN_REMOTE, enums_1.HttpStatus.NOT_FOUND);
        }
        return foundRemoteProject;
    };
    /**
     * This methods builds an URL for the CaaS.
     * Based upon the optional {@link buildCaaSUrlParams buildCaaSUrlParams} object the returning url can link to any desired document.
     * @param id a specific CaaS document id
     * @param locale the locale of CaaS document (id of the document must be set)
     * @param remoteProject name of the remote project
     * @param additionalParams additional URL parameters
     * @param filters filters for CaaS documents - for more details read the [CaaS Platform documentation](https://docs.e-spirit.com/module/caas-platform/CaaS_Platform_Documentation_EN.html#use-of-filters)
     * @param page number of the page you want to access
     * @param pagesize number of the resulting CaaS documents
     * @returns a string that contains the CaaS URL with the configured parameters as query parameters
     */
    FSXARemoteApi.prototype.buildCaaSUrl = function (_a) {
        var _b = _a === void 0 ? {} : _a, id = _b.id, locale = _b.locale, remoteProjectId = _b.remoteProject, additionalParams = _b.additionalParams, filters = _b.filters, page = _b.page, pagesize = _b.pagesize, sort = _b.sort;
        var projectId = this.projectID;
        if (remoteProjectId) {
            this.verifyRemoteProjectExists(remoteProjectId);
            projectId = remoteProjectId;
        }
        var baseURL = this.caasURL + "/" + this.tenantID + "/" + projectId + "." + this.contentMode + ".content";
        var encodedBaseURL = encodeURI(baseURL);
        if (id) {
            encodedBaseURL += "/" + encodeURIComponent(id);
            if (locale) {
                encodedBaseURL += "." + encodeURIComponent(locale);
            }
        }
        var params = [];
        var additionalParamsDefined = additionalParams && Object.keys(additionalParams).length > 0;
        if (additionalParamsDefined && additionalParams) {
            params.push(this.buildStringifiedQueryParams(additionalParams));
        }
        if (filters) {
            var localeFilter = [];
            if (locale) {
                if (typeof locale !== 'string' || !locale.includes('_')) {
                    this._logger.error('[buildNavigationServiceUrl]', "Invalid locale format. Expected format: 'xx_YY' but got '" + locale.toString() + "'");
                    throw new exceptions_1.HttpError(enums_1.FSXAApiErrors.INVALID_LOCALE, enums_1.HttpStatus.BAD_REQUEST);
                }
                localeFilter = [
                    {
                        operator: QueryBuilder_1.ComparisonQueryOperatorEnum.EQUALS,
                        value: locale.split('_')[0],
                        field: 'locale.language',
                    },
                    {
                        operator: QueryBuilder_1.ComparisonQueryOperatorEnum.EQUALS,
                        value: locale.split('_')[1],
                        field: 'locale.country',
                    },
                ];
            }
            var allFilters = __spreadArray(__spreadArray([], filters, true), localeFilter, true);
            // const query = [...filters.map(filter => JSON.stringify(this._queryBuilder.build(filter)))]
            var query = this._queryBuilder
                .buildAll(allFilters)
                .map(function (v) { return encodeURIComponent(JSON.stringify(v)); });
            if (query) {
                params.push('filter=' + query.join('&filter='));
            }
        }
        if (page) {
            params.push('page=' + encodeURIComponent(page));
        }
        if (pagesize) {
            params.push('pagesize=' + encodeURIComponent(pagesize));
        }
        if (sort && sort.length) {
            sort.forEach(function (_a) {
                var name = _a.name, order = _a.order;
                params.push("sort=" + (order === 'desc' ? '-' : '') + encodeURIComponent(name));
            });
        }
        if (params.length) {
            encodedBaseURL += "?" + params.join('&');
        }
        this._logger.info("[buildCaaSUrl] built URL:", encodedBaseURL);
        return encodedBaseURL;
    };
    /**
     * This fuction builds the URL for the NavigationService.
     * Based upon the optional {@link buildNavigationServiceURLParams buildNavigationServiceURLParams} object the returning url can link
     * to a seo route or a locale specific subtree of the navigation.
     * For more details how the Navigation Service works,
     * read the [Navigation Service documentation](https://navigationservice.e-spirit.cloud/docs/user/en/documentation.html).
     * @param locale value must be ISO conform, both 'en' and 'en_US' are valid."
     * @param initialPath can be provided when you want to access a subtree of the navigation
     * @returns {string} the Navigation Service url for either a subtree of or a complete navigation
     */
    FSXARemoteApi.prototype.buildNavigationServiceUrl = function (_a) {
        var _b = _a === void 0 ? {} : _a, locale = _b.locale, initialPath = _b.initialPath, all = _b.all;
        this._logger.debug('[buildNavigationServiceUrl]', { locale: locale, initialPath: initialPath });
        var useInitialPath = initialPath && initialPath !== '/';
        if (locale && useInitialPath) {
            this._logger.warn('[buildNavigationServiceUrl]', 'Parameters "locale" and "initialPath" have been given.');
        }
        var baseNavigationServiceUrl = this.navigationServiceURL + "/" + this.contentMode + "." + this.projectID;
        if (useInitialPath) {
            this._logger.debug('[buildNavigationServiceUrl]', "Using initial path: " + useInitialPath);
            var queryParams = [
                'depth=99',
                '&format=caas',
                "" + (all ? '&all' : ''),
            ].join('');
            return baseNavigationServiceUrl + "/by-seo-route/" + initialPath + "?" + queryParams;
        }
        if (locale) {
            this._logger.debug('[buildNavigationServiceUrl]', "Using locale: " + locale);
            return baseNavigationServiceUrl + "?depth=99&format=caas&language=" + encodeURIComponent(locale);
        }
        return baseNavigationServiceUrl;
    };
    /**
     * This method fetches the navigation from the configured navigation service.
     * The {@link FetchNavigationParams FetchNavigationParams} object defines options for the navigation.
     * Check {@link buildNavigationServiceUrl buildNavigationServiceUrl} to know which URL will be used.
     * @param locale value must be ISO conform, both 'en' and 'en_US' are valid."
     * @param initialPath optional value can be provided when you want to access a subtree of the navigation
     * @param fetchOptions optional object to pass additional request options (Check {@link RequestInit RequestInit})
     * @param filterContext an optional value with additional context used for filtering
     * @returns {Promise<NavigationData | null>} a Promise with the Navigation Service data or null
     */
    FSXARemoteApi.prototype.fetchNavigation = function (_a) {
        var locale = _a.locale, initialPath = _a.initialPath, fetchOptions = _a.fetchOptions, filterContext = _a.filterContext;
        return __awaiter(this, void 0, void 0, function () {
            var encodedInitialPath, forbiddenChars, url, headers, response, _b, _c, _d;
            var _e;
            return __generator(this, function (_f) {
                switch (_f.label) {
                    case 0:
                        this._logger.debug('fetchNavigation', 'start', {
                            locale: locale,
                            initialPath: initialPath,
                            filterContext: filterContext,
                        });
                        encodedInitialPath = undefined;
                        if (initialPath) {
                            forbiddenChars = ['?', '#'];
                            if (forbiddenChars.some(function (char) { return initialPath.includes(char); })) {
                                // error is unknown so that we don't give away how our encoding works
                                this._logger.error('[fetchNavigation] Forbidden char in initial path');
                                throw new exceptions_1.HttpError(enums_1.FSXAApiErrors.UNKNOWN_ERROR, enums_1.HttpStatus.BAD_REQUEST);
                            }
                            encodedInitialPath = encodeURI(initialPath);
                        }
                        if (locale === '') {
                            this._logger.warn('[fetchNavigation] Empty locale given. It is better to not give a locale at all, or provide a valid locale.');
                        }
                        url = this.buildNavigationServiceUrl({
                            initialPath: encodedInitialPath,
                            locale: locale,
                            all: true,
                        });
                        headers = {
                            'Accept-Language': '*',
                        };
                        this._logger.debug('fetchNavigation', 'url', url);
                        return [4 /*yield*/, fetch(url, __assign({ headers: headers }, fetchOptions))];
                    case 1:
                        response = _f.sent();
                        this._logger.debug('fetchNavigation', 'response', response.status);
                        if (!!response.ok) return [3 /*break*/, 3];
                        _c = (_b = this._logger).error;
                        _d = ['Error while fetching navigation'];
                        _e = {
                            url: url,
                            status: response.status
                        };
                        return [4 /*yield*/, response.text()];
                    case 2:
                        _c.apply(_b, _d.concat([(_e.body = _f.sent(),
                                _e)]));
                        switch (response.status) {
                            case enums_1.HttpStatus.NOT_FOUND:
                                throw new exceptions_1.HttpError(enums_1.FSXAApiErrors.NOT_FOUND, enums_1.HttpStatus.NOT_FOUND);
                            default:
                                throw new exceptions_1.HttpError(enums_1.FSXAApiErrors.UNKNOWN_ERROR, enums_1.HttpStatus.BAD_REQUEST);
                        }
                        _f.label = 3;
                    case 3:
                        this._logger.debug('fetchNavigation', 'response ok', response.ok);
                        return [2 /*return*/, this.getFilteredNavigation(response, filterContext)];
                }
            });
        });
    };
    FSXARemoteApi.prototype.getFilteredNavigation = function (response, filterContext) {
        return __awaiter(this, void 0, void 0, function () {
            var navigation, idMap, routes, filteredRoutes, allowedRouteIds, seo, structure, filteredIdMap, filteredNavigation;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this._navigationItemFilter) {
                            return [2 /*return*/, response.json()];
                        }
                        return [4 /*yield*/, response.json()];
                    case 1:
                        navigation = _a.sent();
                        idMap = navigation.idMap;
                        routes = Object.keys(idMap).map(function (route) { return idMap[route]; });
                        this._logger.debug('fetchNavigation', 'getFilteredNavigation', 'unfiltered routes count', routes.length);
                        return [4 /*yield*/, this._navigationItemFilter({
                                navigationItems: routes,
                                filterContext: filterContext,
                            })];
                    case 2:
                        filteredRoutes = _a.sent();
                        this._logger.debug('fetchNavigation', 'getFilteredNavigation', 'filtered routes count', filteredRoutes.length);
                        allowedRouteIds = filteredRoutes.map(function (item) { return item.id; });
                        seo = (0, utils_1.removeFromSeoRouteMap)(navigation.seoRouteMap, allowedRouteIds);
                        structure = (0, utils_1.removeFromStructure)(navigation.structure, allowedRouteIds);
                        filteredIdMap = (0, utils_1.removeFromIdMap)(navigation.idMap, allowedRouteIds);
                        filteredNavigation = __assign(__assign({}, navigation), { idMap: filteredIdMap, seoRouteMap: seo, structure: structure });
                        return [2 /*return*/, filteredNavigation];
                }
            });
        });
    };
    /**
     * This method fetches an element from the configured CaaS.
     * The {@link FetchElementParams FetchElementParams} object defines options to specify your request.
     * Check {@link buildCaaSUrl buildCaaSUrl} to know which URL will be used.
     * @typeParam T optional type parameter you can provide to get a typed CaaS object as result.
     * @param id the CaaS id of the element you want to fetch
     * @param locale value must be ISO conform, both 'en' and 'en_US' are valid
     * @param additionalParams optional additional URL parameters
     * @param remoteProject optional name of the remote project
     * @param fetchOptions optional object to pass additional request options (Check {@link RequestInit RequestInit})
     * @returns {Promise<T>} a Promise with the mapped result
     */
    FSXARemoteApi.prototype.fetchElement = function (_a) {
        var _b, _c;
        var id = _a.id, locale = _a.locale, _d = _a.additionalParams, additionalParams = _d === void 0 ? {} : _d, remoteProject = _a.remoteProject, fetchOptions = _a.fetchOptions, filterContext = _a.filterContext, _e = _a.normalized, normalized = _e === void 0 ? false : _e;
        return __awaiter(this, void 0, void 0, function () {
            var remoteProjectId, _f, items, _g, referenceMap, _h, resolvedReferences;
            return __generator(this, function (_j) {
                switch (_j.label) {
                    case 0:
                        if (remoteProject && !this.remotes[remoteProject]) {
                            throw new exceptions_1.HttpError(enums_1.FSXAApiErrors.UNKNOWN_REMOTE, enums_1.HttpStatus.NOT_FOUND);
                        }
                        locale =
                            remoteProject && this.remotes
                                ? (_b = this.remotes[remoteProject]) === null || _b === void 0 ? void 0 : _b.locale
                                : locale;
                        remoteProjectId = remoteProject
                            ? (_c = this.remotes[remoteProject]) === null || _c === void 0 ? void 0 : _c.id
                            : undefined;
                        return [4 /*yield*/, this.fetchByFilter({
                                filters: [
                                    {
                                        field: 'identifier',
                                        operator: QueryBuilder_1.ComparisonQueryOperatorEnum.EQUALS,
                                        value: id,
                                    },
                                ],
                                additionalParams: additionalParams,
                                remoteProject: remoteProjectId,
                                fetchOptions: fetchOptions,
                                filterContext: filterContext,
                                normalized: true,
                                locale: locale,
                            })];
                    case 1:
                        _f = _j.sent(), items = _f.items, _g = _f.referenceMap, referenceMap = _g === void 0 ? {} : _g, _h = _f.resolvedReferences, resolvedReferences = _h === void 0 ? {} : _h;
                        if (items.length === 0) {
                            throw new exceptions_1.HttpError(enums_1.FSXAApiErrors.NOT_FOUND, enums_1.HttpStatus.NOT_FOUND);
                        }
                        return [2 /*return*/, normalized
                                ? { mappedItems: items, referenceMap: referenceMap, resolvedReferences: resolvedReferences }
                                : (0, MappingUtils_1.denormalizeResolvedReferences)(items, referenceMap, resolvedReferences, remoteProjectId, this._logger)[0]];
                }
            });
        });
    };
    /**
     * This method fetches a filtered page from the configured CaaS.
     * The {@link FetchElementParams FetchElementParams} object defines options to specify your request.
     * Check {@link buildCaaSUrl buildCaaSUrl} to know which URL will be used.
     * Example call:
     *
     * ```typescript
      const englishMedia = await fetchByFilter({
        filters: [
          {
            field: 'fsType',
            value: 'Media',
            operator: ComparisonQueryOperatorEnum.EQUALS,
          },
        ],
        "en_GB",
      })
     * ```
     * @param filters array of {@link QueryBuilderQuery QueryBuilderQuery} to filter you request
     * @param locale value must be ISO conform, both 'en' and 'en_US' are valid
     * @param page optional the number of the page you will get results from `(default = 1)` (must be greater than 0)
     * @param pagesize optional the number of document entries you will get back `(default = 30)` (must be greater than 0)
     * @param sort optional the parameter to sort the results by `(default = [])`.
     * @param additionalParams optional additional URL parameters
     * @param remoteProject optional name of the remote project
     * @param fetchOptions optional object to pass additional request options (Check {@link RequestInit RequestInit})
     * @returns the mapped and filtered response from the CaaS request,
     *    if `additionalParams.keys` are set, the result will be unmapped,
     *    if `data._embedded['rh:doc']` is undefined, the returning result will be the unmapped `data` object
     */
    FSXARemoteApi.prototype.fetchByFilter = function (_a, mapper) {
        var filters = _a.filters, locale = _a.locale, _b = _a.page, page = _b === void 0 ? 1 : _b, _c = _a.pagesize, pagesize = _c === void 0 ? 30 : _c, _d = _a.additionalParams, additionalParams = _d === void 0 ? {} : _d, remoteProjectId = _a.remoteProject, fetchOptions = _a.fetchOptions, filterContext = _a.filterContext, _e = _a.sort, sort = _e === void 0 ? [] : _e, _f = _a.normalized, normalized = _f === void 0 ? false : _f;
        return __awaiter(this, void 0, void 0, function () {
            var url, caasApiResponse, _g, message, ignore_1, data, unmappedItems, remoteProjectLocale, mapperLocale, _h, mappedItems, referenceMap, resolvedReferences;
            var _j;
            return __generator(this, function (_k) {
                switch (_k.label) {
                    case 0:
                        // we need this in order to pass CaaSMapper context
                        if (pagesize < 1) {
                            this._logger.warn("[fetchByFilter] pagesize must be greater than zero! Using fallback of 30.");
                            pagesize = 30;
                        }
                        if (page < 1) {
                            this._logger.warn("[fetchByFilter] page must be greater than zero! Using fallback of 1.");
                            page = 1;
                        }
                        url = this.buildCaaSUrl({
                            filters: filters,
                            additionalParams: __assign(__assign({}, additionalParams), { rep: 'hal' }),
                            remoteProject: remoteProjectId,
                            locale: locale,
                            page: page,
                            pagesize: pagesize,
                            sort: sort,
                        });
                        return [4 /*yield*/, fetch(url, __assign({ headers: this.authorizationHeader }, fetchOptions))];
                    case 1:
                        caasApiResponse = _k.sent();
                        if (!!caasApiResponse.ok) return [3 /*break*/, 8];
                        _g = caasApiResponse.status;
                        switch (_g) {
                            case 401: return [3 /*break*/, 2];
                        }
                        return [3 /*break*/, 3];
                    case 2: throw new exceptions_1.HttpError(enums_1.FSXAApiErrors.NOT_AUTHORIZED, enums_1.HttpStatus.UNAUTHORIZED);
                    case 3:
                        if (!(caasApiResponse.status === 400)) return [3 /*break*/, 7];
                        _k.label = 4;
                    case 4:
                        _k.trys.push([4, 6, , 7]);
                        return [4 /*yield*/, caasApiResponse.json()];
                    case 5:
                        message = (_k.sent()).message;
                        this._logger.error("[fetchByFilter] Bad Request: " + message);
                        return [3 /*break*/, 7];
                    case 6:
                        ignore_1 = _k.sent();
                        return [3 /*break*/, 7];
                    case 7: throw new exceptions_1.HttpError(enums_1.FSXAApiErrors.UNKNOWN_ERROR, enums_1.HttpStatus.BAD_REQUEST);
                    case 8: return [4 /*yield*/, caasApiResponse.json()];
                    case 9:
                        data = _k.sent();
                        unmappedItems = !data._embedded || !data._embedded['rh:doc']
                            ? []
                            : data._embedded['rh:doc'];
                        // if we have no items, we can return early
                        if (unmappedItems.length === 0) {
                            return [2 /*return*/, {
                                    page: 1,
                                    pagesize: 30,
                                    size: undefined,
                                    totalPages: undefined,
                                    items: [],
                                }];
                        }
                        remoteProjectLocale = remoteProjectId
                            ? this.getRemoteConfigById(remoteProjectId).locale
                            : undefined;
                        mapperLocale = locale;
                        if (!mapperLocale && unmappedItems[0].locale) {
                            mapperLocale =
                                unmappedItems[0].locale.language + '_' + unmappedItems[0].locale.country;
                        }
                        mapper =
                            mapper ||
                                new _1.CaaSMapper(this, mapperLocale, {
                                    customMapper: this._customMapper,
                                    maxReferenceDepth: this._maxReferenceDepth,
                                }, new _1.Logger(this._logLevel, 'CaaSMapper'));
                        return [4 /*yield*/, mapper.mapFilterResponse(unmappedItems, additionalParams, filterContext, remoteProjectLocale, remoteProjectId)];
                    case 10:
                        _h = _k.sent(), mappedItems = _h.mappedItems, referenceMap = _h.referenceMap, resolvedReferences = _h.resolvedReferences;
                        if (!this._caasItemFilter) return [3 /*break*/, 12];
                        ;
                        return [4 /*yield*/, this.filterMapResponse(mappedItems, referenceMap, resolvedReferences, filterContext, !!(additionalParams === null || additionalParams === void 0 ? void 0 : additionalParams.keys))];
                    case 11:
                        (_j = _k.sent(), mappedItems = _j.mappedItems, referenceMap = _j.referenceMap, resolvedReferences = _j.resolvedReferences);
                        _k.label = 12;
                    case 12: return [2 /*return*/, __assign(__assign({ page: page, pagesize: pagesize, totalPages: data['_total_pages'], size: data['_size'], items: normalized
                                ? mappedItems
                                : (0, MappingUtils_1.denormalizeResolvedReferences)(mappedItems, referenceMap, resolvedReferences, remoteProjectId, this._logger) }, (normalized && { referenceMap: referenceMap })), (normalized && { resolvedReferences: resolvedReferences }))];
                }
            });
        });
    };
    FSXARemoteApi.prototype.filterMapResponse = function (mappedItems, referenceMap, resolvedReferences, filterContext, isKeysSet) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this._logger.debug('fetchByFilter', 'caasItemFilter is defined, filtering items', mappedItems.map(function (caasItem) {
                            return {
                                type: caasItem.type,
                                id: caasItem.id,
                            };
                        }));
                        return [4 /*yield*/, this._caasItemFilter({
                                mappedItems: mappedItems,
                                referenceMap: referenceMap,
                                resolvedReferences: resolvedReferences,
                                filterContext: filterContext,
                                isKeysSet: isKeysSet,
                            })];
                    case 1: 
                    // _caasItemFilter needs to work with normalized Data
                    return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    // TODO: Fix unecessary array wrapping with a future major jump (as it's breaking)
    /**
     * This method fetches the project properties from the configured CaaS.
     * It uses {@link fetchByFilter fetchByFilter} to get them.
     * @param locale value must be ISO conform, both 'en' and 'en_US' are valid
     * @param additionalParams optional additional URL parameters
     * @param resolve optional array of fsTypes that will be resolved `(default = 'GCAPage')`
     * @param filterContext
     * @param normalized
     * @returns the resolved project properties
     */
    FSXARemoteApi.prototype.fetchProjectProperties = function (_a) {
        var locale = _a.locale, _b = _a.additionalParams, additionalParams = _b === void 0 ? {} : _b, _c = _a.resolve, resolve = _c === void 0 ? ['GCAPage'] : _c, filterContext = _a.filterContext, _d = _a.normalized, normalized = _d === void 0 ? false : _d;
        return __awaiter(this, void 0, void 0, function () {
            var fetchResponse, projectProperties, idToKeyMap, objectKeysToResolve, idsToFetch, _e, resolveItems, resolveReferenceMap, resolveResolvedReferences;
            return __generator(this, function (_f) {
                switch (_f.label) {
                    case 0: return [4 /*yield*/, this.fetchByFilter({
                            filters: [
                                {
                                    field: 'fsType',
                                    value: 'ProjectProperties',
                                    operator: QueryBuilder_1.ComparisonQueryOperatorEnum.EQUALS,
                                },
                            ],
                            locale: locale,
                            additionalParams: additionalParams,
                            filterContext: filterContext,
                            normalized: normalized,
                        })];
                    case 1:
                        fetchResponse = (_f.sent());
                        if (!fetchResponse.items[0])
                            return [2 /*return*/, null];
                        projectProperties = fetchResponse.items[0];
                        if (!projectProperties.data) {
                            this._logger.debug("[fetchProjectProperties] Could not find response data. Project properties might not be defined.");
                            return [2 /*return*/, null];
                        }
                        idToKeyMap = {};
                        objectKeysToResolve = Object.keys(projectProperties.data).filter(function (key) { var _a; return resolve.includes((_a = projectProperties.data[key]) === null || _a === void 0 ? void 0 : _a.referenceType); });
                        idsToFetch = objectKeysToResolve.map(function (key) {
                            idToKeyMap[projectProperties.data[key].referenceId] = key;
                            return projectProperties.data[key].referenceId;
                        });
                        if (idsToFetch.length > 100) {
                            this._logger.warn('[fetchProjectProperties] ProjectProperties contain more than 100 Elements to resolve. Only resolving the first 100!');
                        }
                        return [4 /*yield*/, this.fetchByFilter({
                                locale: locale,
                                filters: [
                                    {
                                        field: 'identifier',
                                        operator: QueryBuilder_1.ComparisonQueryOperatorEnum.IN,
                                        value: idsToFetch,
                                    },
                                ],
                                pagesize: 100,
                                filterContext: filterContext,
                                normalized: normalized,
                            })];
                    case 2:
                        _e = (_f.sent()), resolveItems = _e.items, resolveReferenceMap = _e.referenceMap, resolveResolvedReferences = _e.resolvedReferences;
                        // We need to normalize the data to be able to send it to the proxy api
                        // we need to merge referenceMap, resolvedReferences from those 2 calls
                        // from fetchedElements --> projectProperties
                        if (normalized) {
                            return [2 /*return*/, this.fetchProjectPropertiesNormalized({
                                    fetchResponse: fetchResponse,
                                    projectProperties: projectProperties,
                                    resolveItems: resolveItems,
                                    resolveReferenceMap: resolveReferenceMap,
                                    resolveResolvedReferences: resolveResolvedReferences,
                                    idToKeyMap: idToKeyMap,
                                })];
                        }
                        //Insert fetched Data into projectProperties
                        resolveItems.forEach(function (element) {
                            projectProperties.data[idToKeyMap[element.id]] = element.data;
                        });
                        return [2 /*return*/, projectProperties];
                }
            });
        });
    };
    FSXARemoteApi.prototype.fetchProjectPropertiesNormalized = function (_a) {
        var fetchResponse = _a.fetchResponse, projectProperties = _a.projectProperties, resolveItems = _a.resolveItems, resolveReferenceMap = _a.resolveReferenceMap, resolveResolvedReferences = _a.resolveResolvedReferences, idToKeyMap = _a.idToKeyMap;
        var projectPropertiesResolvedReferences = fetchResponse.resolvedReferences, projectPropertiesReferenceMap = fetchResponse.referenceMap;
        return {
            projectProperties: projectProperties,
            projectPropertiesResolvedReferences: projectPropertiesResolvedReferences,
            projectPropertiesReferenceMap: projectPropertiesReferenceMap,
            resolveItems: resolveItems,
            resolveReferenceMap: resolveReferenceMap,
            resolveResolvedReferences: resolveResolvedReferences,
            idToKeyMap: idToKeyMap,
        };
    };
    /**
     * This method fetches a one-time secure token from the configured CaaS.
     * This token is used to establish the WebSocket connection.
     * @returns the secure token
     */
    FSXARemoteApi.prototype.fetchSecureToken = function () {
        return __awaiter(this, void 0, void 0, function () {
            var url, response, _a, _b, _c, _d, securetoken;
            return __generator(this, function (_e) {
                switch (_e.label) {
                    case 0:
                        url = this.caasURL + "/_logic/securetoken?tenant=" + this.tenantID;
                        this._logger.info('fetchSecureToken', url);
                        return [4 /*yield*/, fetch(url, {
                                headers: this.authorizationHeader,
                            })];
                    case 1:
                        response = _e.sent();
                        if (!!response.ok) return [3 /*break*/, 5];
                        if (!(response.status === enums_1.HttpStatus.NOT_FOUND)) return [3 /*break*/, 2];
                        this._logger.error('fetchSecureToken', enums_1.FSXAApiErrors.NOT_FOUND);
                        throw new exceptions_1.HttpError(enums_1.FSXAApiErrors.NOT_FOUND, enums_1.HttpStatus.NOT_FOUND);
                    case 2:
                        if (!(response.status === enums_1.HttpStatus.UNAUTHORIZED)) return [3 /*break*/, 3];
                        this._logger.error('fetchSecureToken', enums_1.FSXAApiErrors.NOT_AUTHORIZED);
                        throw new exceptions_1.HttpError(enums_1.FSXAApiErrors.NOT_AUTHORIZED, enums_1.HttpStatus.UNAUTHORIZED);
                    case 3:
                        _b = (_a = this._logger).error;
                        _c = ['fetchSecureToken',
                            enums_1.FSXAApiErrors.UNKNOWN_ERROR, response.status + " - " + response.statusText];
                        return [4 /*yield*/, response.text()];
                    case 4:
                        _b.apply(_a, _c.concat([_e.sent()]));
                        throw new exceptions_1.HttpError(enums_1.FSXAApiErrors.UNKNOWN_ERROR, enums_1.HttpStatus.BAD_REQUEST);
                    case 5: return [4 /*yield*/, response.json()];
                    case 6:
                        _d = (_e.sent()).securetoken, securetoken = _d === void 0 ? null : _d;
                        return [2 /*return*/, securetoken];
                }
            });
        });
    };
    FSXARemoteApi.prototype.buildStringifiedQueryParams = function (params) {
        var result = {};
        Object.keys(params).forEach(function (key) {
            if (Array.isArray(params[key])) {
                result[key] = params[key].map(JSON.stringify);
            }
            else if (typeof params[key] === 'object') {
                result[key] = JSON.stringify(params[key]);
            }
            else {
                result[key] = params[key];
            }
        });
        return (0, qs_1.stringify)(result, {
            indices: false,
            encode: true,
            encoder: encodeURIComponent,
        });
    };
    Object.defineProperty(FSXARemoteApi.prototype, "apikey", {
        /**
         * @returns the configured apikey
         */
        get: function () {
            return this._apikey;
        },
        /**
         * This method sets the string value of the apikey.
         * @param value the mandatory string value of the apikey
         */
        set: function (value) {
            if (!value) {
                throw new Error(enums_1.FSXAApiErrors.MISSING_API_KEY);
            }
            this._apikey = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(FSXARemoteApi.prototype, "caasURL", {
        /**
         * @returns the configured CaaS base url
         */
        get: function () {
            return this._caasURL;
        },
        /**
         * This method sets the base url of the CaaS.
         * @param value the mandatory base url of the CaaS e.g. `https://customername-dev-caas-api.e-spirit.cloud`
         */
        set: function (value) {
            if (!value) {
                throw new Error(enums_1.FSXAApiErrors.MISSING_CAAS_URL);
            }
            this._caasURL = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(FSXARemoteApi.prototype, "projectID", {
        /**
         * @returns the configured FirstSpirit project id
         */
        get: function () {
            return this._projectID;
        },
        /**
         * This method sets the FirstSpirit project id
         * @param value the mandatory FirstSpirit project id
         */
        set: function (value) {
            if (!value) {
                throw new Error(enums_1.FSXAApiErrors.MISSING_PROJECT_ID);
            }
            this._projectID = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(FSXARemoteApi.prototype, "tenantID", {
        /**
         * @returns the configured tenant id
         */
        get: function () {
            return this._tenantID;
        },
        /**
         * This method sets the tenant id
         * @param value the mandatory tenant id (schema: `customername-stage`)
         */
        set: function (value) {
            if (!value) {
                throw new Error(enums_1.FSXAApiErrors.MISSING_TENANT_ID);
            }
            this._tenantID = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(FSXARemoteApi.prototype, "navigationServiceURL", {
        /**
         * @returns the configured NavigationService url
         */
        get: function () {
            return this._navigationServiceURL;
        },
        /**
         * This method sets the url of the NavigationService.
         * @param value the mandatory url of the NavigationService e.g. `https://customername-stage-navigationservice.e-spirit.cloud/navigation`
         */
        set: function (value) {
            if (!value) {
                throw new Error(enums_1.FSXAApiErrors.MISSING_NAVIGATION_SERVICE_URL);
            }
            this._navigationServiceURL = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(FSXARemoteApi.prototype, "remotes", {
        /**
         * @returns the configured remote project configuration
         */
        get: function () {
            return this._remotes;
        },
        /**
         * This method sets the remote project configuration
         * @param value the mandatory remote project configuration
         * example:
         * ```typescript
          {
            "media": {
              "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
              "locale": "en_GB"
            }
          }
         ```
         */
        set: function (value) {
            var keys = Object.keys(value);
            keys.forEach(function (key) {
                var _a = value[key], id = _a.id, locale = _a.locale;
                if (!id) {
                    throw new Error(enums_1.FSXAApiErrors.MISSING_REMOTE_ID);
                }
                if (!locale) {
                    throw new Error(enums_1.FSXAApiErrors.MISSING_REMOTE_LOCALE);
                }
            });
            this._remotes = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(FSXARemoteApi.prototype, "contentMode", {
        /**
         * @returns the configured content mode
         * `preview` links to the unreleased content
         * `release` links to the published content
         */
        get: function () {
            return this._contentMode;
        },
        /**
         * This method sets the content mode
         * @param value the mandatory value of the content mode (allowed: `preview`, `release`)
         */
        set: function (value) {
            if (value !== 'preview' && value !== 'release') {
                throw new Error(enums_1.FSXAApiErrors.UNKNOWN_CONTENT_MODE);
            }
            this._contentMode = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(FSXARemoteApi.prototype, "logLevel", {
        /**
         * @returns the configured log level
         */
        get: function () {
            return this._logLevel;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Getter/Setter to enable the CaaS event stream
     * @returns true, if a event stream should pipe events from CaaS change events websocket
     */
    FSXARemoteApi.prototype.enableEventStream = function (enable) {
        if (typeof enable !== 'undefined')
            this._enableEventStream = enable;
        return this._enableEventStream;
    };
    return FSXARemoteApi;
}());
exports.FSXARemoteApi = FSXARemoteApi;
//# sourceMappingURL=FSXARemoteApi.js.map