"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
var saxes = __importStar(require("saxes"));
var lodash_1 = require("lodash");
var utils_1 = require("../utils");
// map characters which are not valid inside XML attributes enclosed in double quotes to their
// entity representation
var ENTITIES = new Map([
    ['"', '&quot;'],
    ['&', '&amp;'],
    ['<', '&lt;'],
    ['>', '&gt;'],
    ["'", 'apos;'],
]);
var XMLParser = /** @class */ (function () {
    function XMLParser(logger) {
        this.getCurrentElement = function (result, path) {
            return path.length === 0 ? result : (0, lodash_1.get)(result, path, null);
        };
        this.logger = logger;
    }
    XMLParser.prototype.sanitizeXml = function (xml) {
        try {
            return (0, utils_1.replaceUnnestedLinkTemplatesWithOneLinkElement)(xml
                .replace(/&nbsp;/g, '&#160;')
                // replace all non closing br tags with self-closing once (legacy, fixed with CORE-13424)
                .replace(/<br>/g, '<br />'), 
            // restructure the link structure into one single link element  (hint: *? matches non-eager)
            ENTITIES);
        }
        catch (err) {
            this.logger.error('[XMLParser]: Error sanitizing the xml', err, xml);
            return '';
        }
    };
    XMLParser.prototype.parse = function (xml) {
        var _this = this;
        var sanitizedXml = this.sanitizeXml(xml);
        return new Promise(function (resolve) {
            // this will be the result, wrapped with a "root" node
            var result = {
                data: {},
                type: 'root',
                content: [],
            };
            // we will track the current nested node through the path property
            var path = [];
            var parser = new saxes.SaxesParser(false);
            parser.on('error', function (err) {
                _this.logger.error('[XMLParser]: Error parsing XML', err, xml);
            });
            parser.on('end', function () {
                resolve(result.content[0].content);
            });
            parser.on('text', function (text) {
                var parent = _this.getCurrentElement(result, path);
                if (!parent || !Array.isArray(parent.content))
                    return;
                parent.content.push({
                    type: 'text',
                    content: text,
                    data: {},
                });
            });
            parser.on('opentag', function (tag) {
                var parent = _this.getCurrentElement(result, path);
                if (!parent || !Array.isArray(parent.content))
                    return;
                var elementIndex = parent.content.push(_this.createRichTextElement(tag));
                path = __spreadArray(__spreadArray([], path, true), ['content', elementIndex - 1 + ''], false);
            });
            parser.on('closetag', function (tag) {
                path.splice(-2, 2);
            });
            // we will sanitize the incoming xml and wrap it with a root node, so we can be sure, that there will only be one root - node
            // this node will not be returned in the result
            parser.write("<root>" + sanitizedXml + "</root>").close();
        });
    };
    XMLParser.prototype.createRichTextElement = function (tag) {
        switch (tag.name) {
            case 'b':
                return {
                    data: __assign({ format: 'bold' }, tag.attributes),
                    content: [],
                    type: 'text',
                };
            case 'i':
                return {
                    data: __assign({ format: 'italic' }, tag.attributes),
                    content: [],
                    type: 'text',
                };
            case 'ul':
                return {
                    data: tag.attributes,
                    content: [],
                    type: 'list',
                };
            case 'li':
                return {
                    data: tag.attributes,
                    content: [],
                    type: 'listitem',
                };
            case 'br':
                return {
                    data: tag.attributes,
                    content: [],
                    type: 'linebreak',
                };
            case 'p':
                return {
                    data: tag.attributes,
                    content: [],
                    type: 'paragraph',
                };
            case 'div':
                return {
                    data: tag.attributes,
                    content: [],
                    type: 'block',
                };
            case 'link':
                var data = tag.attributes;
                if (data.data) {
                    try {
                        data.data = JSON.parse(data.data);
                    }
                    catch (err) {
                        this.logger.error('[XMLParser]: Error parsing data-attribute of link-element', data.data);
                    }
                }
                return {
                    data: data,
                    content: [],
                    type: 'link',
                };
            default:
                return {
                    data: tag.attributes,
                    content: [],
                    type: tag.name,
                };
        }
    };
    return XMLParser;
}());
exports.default = XMLParser;
//# sourceMappingURL=XMLParser.js.map