import { FSXARemoteApi } from '../modules';
import { FetchByFilterBody, FetchElementRouteBody, FetchNavigationRouteBody, FetchProjectPropertiesBody } from '../routes';
import { ParamValidationError } from './parameterValidation';
import { FetchResponse, NavigationData, NormalizedProjectPropertyResponse, ProjectProperties } from '../types';
export declare type FetchWrapperResult<T = any> = FetchWrapperSuccess<T> | FetchWrapperError;
declare type FetchWrapperSuccess<T = any> = {
    success: true;
    data: T;
};
declare type FetchWrapperError = {
    success: false;
    error: {
        statusCode: number;
        message: string;
    };
    originalError: Error | ParamValidationError;
};
/**
 * Useful Error Messages for Validation Errors.
 * Can be used as an Error Message to send to the client
 */
export declare const PARAM_VALIDATION_ERROR_TO_MESSAGE: Record<ParamValidationError['reason'], string>;
/**
 * Provides Wrapper Functions fot the given api.
 * @param api FSXAApi to wrap
 * @param loggerName Creates a logger with the given name
 * @returns
 */
export declare const useEndpointIntegrationWrapper: (api: FSXARemoteApi, loggerName?: string) => {
    fetchElementWrapper: (params: FetchElementRouteBody, logContext?: string) => Promise<FetchWrapperResult>;
    fetchNavigationWrapper: (params: FetchNavigationRouteBody, logContext?: string) => Promise<FetchWrapperResult<NavigationData | null>>;
    fetchByFilterWrapper: (params: FetchByFilterBody, logContext?: string) => Promise<FetchWrapperResult<FetchResponse>>;
    fetchProjectPropertiesWrapper: (params: FetchProjectPropertiesBody, logContext?: string) => Promise<FetchWrapperResult<ProjectProperties | NormalizedProjectPropertyResponse | null>>;
};
export {};
